@@ -157,29 +157,38 @@ conversation(int num_msg, const struct pam_message **msg,
         (*resp)[i].resp_retcode=0;
      }
    for (i=0; (i<num_msg) && (ret==PAM_SUCCESS); ++i) 
-     {
-        if (!(stage=which_stage(conv_data->prompts,msg[i]->msg)))
-          {
-             if (conv_data->stage) 
-               error_msg(stderr,"unregistered password prompts got from PAM:",
-                         0,-1);
-             fprintf (stderr,"  %s\n",msg[i]->msg);
-             ret=PAM_CONV_ERR;
-             conv_data->stage=0;
-          }
-        else if ((stage<conv_data->stage) || !(answer=
-             (*conv_data->ask_pass)(conv_data->data,msg[i]->msg,stage)))
-          {
-             ret=PAM_CONV_ERR;
-             break;
-          }
-        else if (ret==PAM_SUCCESS)
-          {
-             (*resp)[i].resp=(char *)x_strdup(answer);
-             _pam_overwrite(answer);
-             conv_data->stage=stage+1;
-          }
-     }
+     switch (msg[i]->msg_style)
+       {
+        case PAM_ERROR_MSG:
+          error_msg(stderr,"error message got from PAM:",
+                    0,-1);
+          fprintf (stderr,"  %s\n",msg[i]->msg);
+          ret=PAM_CONV_ERR;
+          conv_data->stage=0;
+          break;
+        case PAM_TEXT_INFO:
+          fprintf (stdout,"  %s\n",msg[i]->msg);
+          break;
+        default:
+          if (!(stage=which_stage(conv_data->prompts,msg[i]->msg)))
+            {
+               if (conv_data->stage) 
+                 error_msg(stderr,"unregistered password prompts got from PAM:",
+                           0,-1);
+               fprintf (stderr,"  %s\n",msg[i]->msg);
+               ret=PAM_CONV_ERR;
+               conv_data->stage=0;
+            }
+          else if ((stage<conv_data->stage) || !(answer=
+                   (*conv_data->ask_pass)(conv_data->data,msg[i]->msg,stage)))
+            ret=PAM_CONV_ERR;
+          else if (ret==PAM_SUCCESS)
+            {
+              (*resp)[i].resp=(char *)x_strdup(answer);
+              _pam_overwrite(answer);
+              conv_data->stage=stage+1;
+            }
+       }
    return ret;
 }
 
